using System;
using System.Configuration;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Threading;

using KnownObjects;
using Belikov.GenuineChannels;
using Belikov.GenuineChannels.BroadcastEngine;
using Belikov.GenuineChannels.DotNetRemotingLayer;
using Belikov.GenuineChannels.GenuineUdp;
using Belikov.GenuineChannels.Logbook;

namespace Client
{
	/// <summary>
	/// ChatClient demostrates simple client application.
	/// </summary>
	class ChatClient
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			// setup .NET Remoting
			Console.WriteLine("Configuring Remoting environment...");
			System.Configuration.ConfigurationSettings.GetConfig("DNS");
			GenuineGlobalEventProvider.GenuineChannelsGlobalEvent += new GenuineChannelsGlobalEventHandler(GenuineChannelsEventHandler);
			RemotingConfiguration.Configure("Client.exe.config");
			//GenuineGlobalLoggerContainer.Logger = new BinaryLog(@"c:\tmp\client.log", false);

			Console.WriteLine(".NET Remoting has been configured from Client.exe.config file.");
			Console.WriteLine("Enter a nickname of this instance:");

			// bind client's receiver to the known court
			RemotingServices.Marshal(new NicknameProvider(Console.ReadLine()), "AskNickname.rem");
			CourtCollection.Attach("/test/chat", new MessageReceiver(), null);

			// and set up IP multicast sender
			Dispatcher dispatcher = new Dispatcher(typeof(IMessageReceiver));
			IBroadcastSenderProvider iBroadcastSenderProvider = (IBroadcastSenderProvider) ChannelServices.GetChannel("IPMulticastSender");
			dispatcher.Add(iBroadcastSenderProvider.GetBroadcastSender("/test/chat"));
			IMessageReceiver iMessageReceiver = (IMessageReceiver) dispatcher.TransparentProxy;

			try
			{
				for(;;)
				{
					// ask a string
					Console.WriteLine("Enter a message to send or an empty string to exit.");
					string str = Console.ReadLine();
					if (str.Length <= 0)
						return ;

					// send a message to all clients receiving to 11000 UDP port
					iMessageReceiver.ReceiveMessage(str);
				}
			}
			catch(Exception ex)
			{
				Console.WriteLine("Exception: {0}. Stack trace: {1}.", ex.Message, ex.StackTrace);
			}
		}

		public static void GenuineChannelsEventHandler(object sender, GenuineEventArgs e)
		{
			if (e.SourceException == null)
				Console.WriteLine("Global event: {0}\r\nUrl: {1}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString());
			else
				Console.WriteLine("Global event: {0}\r\nUrl: {1}\r\nException: {2}", e.EventType, 
					e.HostInformation == null ? "<not specified>" : e.HostInformation.ToString(), 
					e.SourceException);
		}

	}
}
