using System;
using System.Collections;

using KnownObjects;
using Belikov.GenuineChannels;
using Belikov.GenuineChannels.Security;

namespace Client
{
	/// <summary>
	/// MessageReceiver.
	/// </summary>
	public class MessageReceiver : MarshalByRefObject, IMessageReceiver
	{
		/// <summary>
		/// Nicknames.
		/// </summary>
		private Hashtable _nicknames = Hashtable.Synchronized(new Hashtable());

		/// <summary>
		/// Is called by the server when a message is accepted.
		/// </summary>
		/// <param name="message">A message.</param>
		/// <returns>Null.</returns>
		public object ReceiveMessage(string message)
		{
			// try to fetch senders nickname by senders IP address
			string remoteIPAddress = GenuineUtility.CurrentRemoteHost.PhysicalAddress.ToString();
			string remoteNickname = _nicknames[remoteIPAddress] as string;
			if (remoteNickname == null)
				remoteNickname = "Unknown. Asking...";

			// show a message
			Console.WriteLine("\r\nMessage \"{0}\" has been received from \"{1}\" (IP address: {2}).", message, 
				remoteNickname, remoteIPAddress);

			// check whether we know its nickname
			if (! _nicknames.ContainsKey(remoteIPAddress))
			{
				IAskNickname iAskNickname = (IAskNickname) Activator.GetObject(typeof(IAskNickname),
					GenuineUtility.CurrentRemoteUri + "/AskNickname.rem");

				try
				{
					// ask a nickname with a 5 seconds reply timeout
					SecuritySessionParameters parameters = new SecuritySessionParameters(SecuritySessionServices.DefaultContext.Name, SecuritySessionAttributes.None, TimeSpan.FromSeconds(5));
					using(new SecurityContextKeeper(parameters))
						this._nicknames[remoteIPAddress] = iAskNickname.GetNickname();

					Console.WriteLine("Name resolved. It was {0}.", this._nicknames[remoteIPAddress]);
				}
				catch(Exception ex)
				{
					Console.WriteLine("Can't receive remote host name due to exception: {0}.", ex.Message);
					this._nicknames[remoteIPAddress] = "Unknown";
				}
			}

			return null;
		}

		/// <summary>
		/// This is to insure that when created as a Singleton, the first instance never dies,
		/// regardless of the expired time.
		/// </summary>
		/// <returns></returns>
		public override object InitializeLifetimeService()
		{
			return null;
		}
	}
}
